#!/usr/bin/python3.6
from src.imports import *
from src.utils import get_config

config = get_config()
webhook = config["teams"]["webhook"]

@click.group()
def cli():
    pass

@cli.command()
def getProjects():
    selected_project = questionary.select(
        "Please select the project you'll be working on",
        choices=jira_client.get_projects_names()
    ).ask()
    return selected_project

@cli.command()
def getIssues():
    #! For this command to work you need to manually add the project variable
    #! Or use the getProjects function
    project = ''
    selected_issue = questionary.select(
        "Please select the issue you'll be working on",
        choices=jira_client.get_issues_project(project)
    ).ask()
    click.echo(selected_issue)

@cli.command()
def sendMessage():
    try:
        # * You must create the connectorcard object with the Microsoft Webhook URL
        myTeamsMessage = pymsteams.connectorcard(webhook)

        # * Add text to the message.
        myTeamsMessage.text("This message was sent using Leona")

        # * create the section
        myMessageSection = pymsteams.cardsection()

        # * Section Title
        myMessageSection.title("Feature started")

        # * Activity Elements
        myMessageSection.activityTitle(
            "User " + " started working on the feature: issue_key")
        myMessageSection.activitySubtitle("my activity subtitle")
        myMessageSection.activityImage(
            "https://img.favpng.com/24/25/19/github-repository-version-control-source-code-png-favpng-C3CZbFLHhkc9CB2Qu0CbHEY4W.jpg")
        myMessageSection.activityText("This is my activity Text")

        # * Facts are key value pairs displayed in a list.

        # * Add your section to the connector card object before sending
        myTeamsMessage.addSection(myMessageSection)
        # * send the message.
        myTeamsMessage.send()
        click.echo('Message sent succesfully')
    except Exception as e:
        click.echo('Something Failed : {0}'.format(e))

@cli.command()
def pr():
    # !This code gets all the default reviewers of a specific repository
    # !In this case the repository is leona
    reviewers_name, reviewers_uuid = bitbucket_client.get_reviewers('leona')
    title_prompt = (Input(prompt="Enter the title of the pull request: "))
    description_prompt = (Input(prompt="Enter the description of the pull request: "))
    reviewers_prompt = (Check(prompt="Select the reviewers: ", choices = reviewers_name))
    title_result = title_prompt.launch()
    description_result = description_prompt.launch()
    reviewers_result = reviewers_prompt.launch()
    formated_uuid = bitbucket_client.get_formated_uuid(reviewers_result,reviewers_name, reviewers_uuid)
    branch_name = bitbucket.client.get_current_branch_name()    
    repo_name = bitbucket_client.get_current_repo_name()
    response = bitbucket_client.create_pull_request(title_result, description_result, branch_name, formated_uuid, repo_name)
    click.echo('Pull Request Created succesfully')

@cli.command()
def commitsLog():
    repo_names = ['flowy_iac','flowy_gizmo_configurator_web', 'flowy_api_ts','flowy_form_mobile','flowy_core','flowy_form_web',
                  'flowy_analytics_api','flowy_developer_documentation','flowy_api','flowy_agent','flowy_analytics_web']
    authors, messages, dates, diffs, diffs_data, diffs_stats_data, hashs, lines_added, lines_removed = ([] for i in range(9))
    selected_repo = questionary.select(
        "Please select the issue you'll be working on",
        choices=repo_names
    ).ask()
    
    commits_response = bitbucket_client.get_commits(selected_repo)
    for commit in commits_response['values']:
        authors.append(commit['author']['user']['display_name'])
        messages.append(commit['message'])
        dates.append(commit['date'])
        diffs.append(commit['links']['diff']['href'])
        hashs.append(commit['hash'])
    
    diffs_data = bitbucket_client.get_diffs_data(diffs)
    lines_added, lines_removed = bitbucket_client.get_diffs_stats(hashs,selected_repo)
    log = [{'author': a, 'message': b, 'date': c, 'diff': d, 'lines_added': e, 'lines_removed': f}
           for (a, b, c, d, e, f) in zip(authors, messages, dates, diffs_data, lines_added, lines_removed)]
    mylog = DataFrame(log)
    writer = ExcelWriter('%s_commit_logs.xlsx' % selected_repo)
    mylog.to_excel(writer)
    writer.close()
    click.echo('The excel file has been created succesfully!')

@cli.command()
def jiraLogs():
    projects, keys, issue_name, issue_status, issue_description, issue_summary, issue_assignee, issue_updated = ([] for i in range(8))
   
    selected_project = questionary.select(
        "Please select the project you'll be working on",
        choices=jira_client.get_projects_names()
    ).ask()
        
    issues_data = jira_client.get_all_issues(selected_project, ["key", "summary", "status", "assignee", "updated"])
    
    for issue in issues_data:
        issue_name.append(issue.key)
        issue_status.append(issue.fields.status.name)
        issue_description.append(issue.fields.status.description)
        issue_summary.append(issue.fields.summary)
        issue_assignee.append(str(issue.fields.assignee))
        issue_updated.append(issue.fields.updated)

    log = [{'name': i, 'description': j, 'summary': v, 'assignee': x, 'updated': y,'status': z}
           for (i, j, v, x, y, z) in zip(issue_name, issue_description, issue_summary, issue_assignee, issue_updated,issue_status)]
    
    mylog = DataFrame(log)
    writer = ExcelWriter('%s_jira_logs.xlsx' % selected_project, engine='xlsxwriter')
    mylog.to_excel(writer, sheet_name='logs')
    writer.close()
    click.echo('The excel file has been created succesfully!')

@cli.command()
@click.option('-r','--registry')
@click.option('-t','--tag')
@click.option('-v','--version')
@click.option('-d','--dockerfile')
def build(registry,tag,version,dockerfile):
    command = 'docker build --no-cache -t %s/%s:%s -f %s .' % (registry,tag,version,dockerfile)
    os.system(command)

@cli.command()
@click.option('-r','--registry')
@click.option('-t','--tag')
@click.option('-v','--version')
@click.option('-d','--dockerfile')
def build_prod(registry,tag,version,dockerfile):
    command = 'docker build --no-cache -t %s/%s:%s -f %s .' % (registry,tag,version,dockerfile)
    os.system(command)

@cli.command()
@click.option('-r','--registry')
@click.option('-p','--repo_name')
@click.option('-t','--tag')
def push(registry,repo_name,tag):
    command = 'docker push %s/%s:%s' % (registry,repo_name,tag)
    os.system(command)

@cli.command()
@click.option('-r','--registry')
@click.option('-t','--tag')
@click.option('-v','--version')
def test(registry, tag, version):
    command = """docker run %s/%s:%s bash -c 'yarn test-ci'""" % (registry, tag, version)
    os.system(command)

@cli.command()
@click.option('-p','--password')
@click.option('-s','--service')
def deploy(password, service):
    command = """ANSIBLE_HOST_KEY_CHECKING=false ansible-playbook /etc/ansible/playbook.yml -i /etc/ansible/hosts --extra-vars 'ansible_sudo_pass=%s' -e service_name=%s""" % (password, service)
    click.echo(command)
    os.system(command)


import os
import yaml

DEFAULT_CONFIG_PATH = os.path.join(os.getcwd(), "static/CONFIG_TEMPLATE.yml")
OUT_FILE_PATH = os.path.join(os.path.expanduser("~"), "leona_config.yml")

def get_config():
    if (os.path.exists(OUT_FILE_PATH) == False):
        create_config()
        config_file_path = os.path.join(os.path.expanduser("~"), "leona_config.yml")
        with open(config_file_path) as config_file:
            config = yaml.load(config_file, Loader=yaml.FullLoader)
        return config
    else:
        config_file_path = os.path.join(os.path.expanduser("~"), "leona_config.yml")
        with open(config_file_path) as config_file:
            config = yaml.load(config_file, Loader=yaml.FullLoader)
        return config
  
def create_config():
    with open(DEFAULT_CONFIG_PATH) as default_config:
        config = yaml.load(default_config, Loader=yaml.FullLoader)
        output_file = yaml.dump(config)
        with open(OUT_FILE_PATH, "w") as final_config:
            final_config.write(output_file)


from requests.auth import HTTPBasicAuth
from os import environ
import click
import os
import sys
import requests
import json
import pymsteams
import logging
import questionary
import os
import subprocess
from pandas import DataFrame, ExcelWriter
import xlsxwriter
from src import bitbucket_client
from src import jira_client
from bullet import Bullet, Check, YesNo, Input, VerticalPrompt



from jira import JIRA
from src.imports import os
from src.imports import environ
from src.utils import get_config

config = get_config()
jira_user = config["jira"]["username"]
jira_token = config["jira"]["token"]
base_url = 'https://heliconjira.atlassian.net'
options = {'server': base_url}
jira = JIRA(options, basic_auth=(jira_user, jira_token))
#! JIRA
def get_projects_names():
  names = []
  for project in jira.projects():
      names.append(project.name)
  return names

def get_projects_keys():
  keys = []
  for project in jira.projects():
      keys.append(project.key)
  return keys

def get_issues_project(project):
  issues = []
  jql = 'project =%s AND status not in (Closed, Resolved, Done) AND assignee=currentuser()' % project
  for issue in jira.search_issues(jql_str= jql,maxResults=200):
    issues.append(issue.key)
  return issues

def get_all_issues(project_name, fields):
    issues = []
    i = 0
    chunk_size = 100
    while True:
        chunk = jira.search_issues(f'project = {project_name}', startAt=i, maxResults=chunk_size, fields=fields)
        i += chunk_size
        issues += chunk.iterable
        if i >= chunk.total:
            break
    return issues


import os
from codecs import open
from setuptools import setup, find_packages

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("requirements.txt", "r") as f:
    requires = f.read()

setup(
    name="leona",
    version='1.0.3',
    author="Juan Guillermo Escalona",
    author_email="j.escalona@hlcn.mx",
    url="https://bitbucket.org/hlcn/leona",
    description='CLI for DevOps Integration',
    long_description=long_description,
    long_description_content_type='text/markdown',
    py_modules=['leona'],
    zip_safe=False,
    packages=find_packages(),
    data_files=None,
    python_requires='>=3.7.6',
    include_package_data=True,
    install_requires=requires,
    entry_points={
        'console_scripts': [
            'leona=src.leona:cli',
        ],
    },
)



#!/usr/bin/python3.6
import os
import requests
import json
import subprocess
from os import environ
from os.path import basename
from requests.auth import HTTPBasicAuth
from src.utils import get_config

config = get_config()
bit_user = config["bitbucket"]["username"]
bit_pass = config["bitbucket"]["password"]
base_url = 'https://api.bitbucket.org/2.0/repositories/hlcn'
auth = HTTPBasicAuth(bit_user, bit_pass)  
headers = {"Accept": "application/json"}

#! BITBUCKET    
def get_commits(repo_name):
    url = '%s/%s/commits' % (base_url,repo_name)
    response = requests.request("GET",url,headers=headers,auth=auth)
    commits = json.loads(response.text)
    return commits

def create_pull_request(title, description, branch, reviewers, repo_name):
    if len(reviewers) == 0:
        json_data = {
            "title": title,
            "description": description,
            "destination": {
                "branch": {
                    "name": "develop"
                }
            },
            "source": {
                "branch": {
                    "name": branch
                }
            }
        }
    else:
        json_data = {
            "title": title,
            "description": description,
            "destination": {
                "branch": {
                    "name": "develop"
                }
            },
            "source": {
                "branch": {
                    "name": branch
                }
            },
            "reviewers": reviewers
        }
    url = "%s/%s/pullrequests" % (base_url,repo_name)
    response = requests.post(url,json=json_data,headers=headers,auth=auth)
    json_data = json.dumps(response.text)
    return json_data
def get_current_repo_name():
    command = subprocess.run(['git', 'rev-parse', '--show-toplevel'], stdout=subprocess.PIPE)
    command = command.stdout.decode('utf-8')
    repo_name = basename(command)
    return repo_name

def get_current_branch_name():
    command = subprocess.run(['git', 'rev-parse', '--abbrev-ref', 'HEAD'], stdout=subprocess.PIPE)
    branch_name = command.stdout.decode('utf-8')
    return branch_name

def get_reviewers(repo_name):
    reviewers_uuid, reviewers_name = ([] for i in range(2))
    url = '%s/%s/default-reviewers' % (base_url ,repo_name)
    response = requests.request("GET",url,headers=headers,auth=auth)
    reviewers_response = json.loads(response.text)
    for reviewer in reviewers_response['values']:
        reviewers_name.append(reviewer['display_name'])
        reviewers_uuid.append(reviewer['uuid'])
    return reviewers_name, reviewers_uuid

def get_formated_uuid(reviewers_result, reviewers_name, reviewers_uuid):
    formated = []
    reviewers_dict = [{'name': i, 'uuid': j} for (i, j) in zip(reviewers_name, reviewers_uuid)]
    for reviewer in reviewers_dict:
        for reviewer_name in reviewers_result:
            if reviewer_name == reviewer['name']:
                formated.append(reviewer['uuid'])
    formated = [{'uuid': i} for (i) in zip(formated)]
    formated_uuid = str(formated).replace("(","").replace(",)", "")
    return formated_uuid

def get_diffs_data(diffs):
    diffs_data = []
    for diff in diffs:
        response = requests.request("GET",diff,headers=headers,auth=auth)
        diffs_data.append(response.text)
    return diffs_data

def get_diffs_stats(hashs,repo):
    lines_added = 0
    lines_removed = 0
    diff_stats, response_data, added, removed = ([] for i in range(4))
    for hashs_item in hashs:
        url = '%s/%s/diffstat/%s' % (base_url,repo, hashs_item)
        response = requests.request("GET",url,headers=headers,auth=auth).json()
        for item in response['values']:
            if 1 < len(response['values']):
                lines_added += item['lines_added']
                lines_removed += item['lines_removed']    
            else:
                lines_added = item['lines_added']
                lines_removed = item['lines_removed']
            added.append('lines added: %s' % lines_added)
            removed.append('lines removed: %s' % lines_removed)
    return added, removed